@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import division, absolute_import, print_function, unicode_literals
+from datetime import datetime
+import re
 import requests
 import json
 import logging
@@ -414,8 +416,8 @@ class API(object):
             "x-emby-authorization": auth
         }
 
-    def send_request(self, url, path, method="get", timeout=None, headers=None, data=None):
-        request_method = getattr(requests, method.lower())
+    def send_request(self, url, path, method="get", timeout=None, headers=None, data=None, session=None):
+        request_method = getattr(session or requests, method.lower())
         url = "%s/%s" % (url, path)
         request_settings = {
             "timeout": timeout or self.default_timeout,
@@ -485,3 +487,72 @@ class API(object):
         url = response.url.replace('/system/info/public', '')
         return url
 
+    #################################################################################################
+
+    # Syncplay
+
+    #################################################################################################
+
+    def _parse_precise_time(self, time):
+        # We have to remove the Z and the least significant digit.
+        return datetime.strptime(time[:-2], "%Y-%m-%dT%H:%M:%S.%f")
+
+    def utc_time(self):
+        # Measure time as close to the call as is possible.
+        server_address = self.config.data.get("auth.server")
+        session = self.client.session
+        
+        response = self.send_request(server_address, "GetUTCTime", session=session)
+        response_received = datetime.utcnow()
+        request_sent = response_received - response.elapsed
+
+        response_obj = response.json()
+        request_received = self._parse_precise_time(response_obj["RequestReceptionTime"])
+        response_sent = self._parse_precise_time(response_obj["ResponseTransmissionTime"])
+
+        return {
+            "request_sent": request_sent,
+            "request_received": request_received,
+            "response_sent": response_sent,
+            "response_received": response_received
+        }
+    
+    def get_sync_play(self, item_id=None):
+        params = {}
+        if item_id is not None:
+            params["FilterItemId"] = item_id
+        return self._get("SyncPlay/List", params)
+
+    def join_sync_play(self, group_id):
+        return self._post("SyncPlay/Join", {
+            "GroupId": group_id
+        })
+
+    def leave_sync_play(self):
+        return self._post("SyncPlay/Leave")
+    
+    def play_sync_play(self):
+        return self._post("SyncPlay/Play")
+
+    def pause_sync_play(self):
+        return self._post("SyncPlay/Pause")
+
+    def seek_sync_play(self, position_ticks):
+        return self._post("SyncPlay/Seek", {
+            "PositionTicks": position_ticks
+        })
+    
+    def buffering_sync_play(self, when, position_ticks, done=False):
+        return self._post("SyncPlay/Buffering", {
+            "When": when.isoformat() + "Z",
+            "PositionTicks": position_ticks,
+            "BufferingDone": done
+        })
+
+    def ping_sync_play(self, ping):
+        return self._post("SyncPlay/Ping", {
+            "Ping": ping
+        })
+
+    def new_sync_play(self):
+        return self._post("SyncPlay/New")
