@@ -9,6 +9,8 @@ import threading
 
 import websocket
 
+from .keepalive import KeepAlive
+
 ##################################################################################################
 
 LOG = logging.getLogger('JELLYFIN.' + __name__)
@@ -17,15 +19,23 @@ LOG = logging.getLogger('JELLYFIN.' + __name__)
 
 
 class WSClient(threading.Thread):
+    multi_client = False
+    global_wsc = None
+    global_stop = False
 
-    wsc = None
-    stop = False
-
-    def __init__(self, client):
+    def __init__(self, client, allow_multiple_clients=False):
 
         LOG.debug("WSClient initializing...")
 
         self.client = client
+        self.keepalive = None
+        self.wsc = None
+        self.stop = False
+        self.message_ids = set()
+
+        if self.multi_client or allow_multiple_clients:
+            self.multi_client = True
+
         threading.Thread.__init__(self)
 
     def send(self, message, data=""):
@@ -49,8 +59,13 @@ class WSClient(threading.Thread):
                                           on_message=lambda ws, message: self.on_message(ws, message),
                                           on_error=lambda ws, error: self.on_error(ws, error))
         self.wsc.on_open = lambda ws: self.on_open(ws)
+        
+        if not self.multi_client:
+            if self.global_wsc is not None:
+                self.global_wsc.close()
+            self.global_wsc = self.wsc
 
-        while not self.stop:
+        while not self.stop and not self.global_stop:
 
             self.wsc.run_forever(ping_interval=10)
 
@@ -71,7 +86,27 @@ class WSClient(threading.Thread):
     def on_message(self, ws, message):
 
         message = json.loads(message)
+
+        # If a message is received multiple times, ignore repeats.
+        message_id = message.get("MessageId")
+        if message_id is not None:
+            if message_id in self.message_ids:
+                return
+            self.message_ids.add(message_id)
+
         data = message.get('Data', {})
+
+        if message['MessageType'] == "ForceKeepAlive":
+            self.send("KeepAlive")
+            if self.keepalive is not None:
+                self.keepalive.stop()
+            self.keepalive = KeepAlive(data, self)
+            self.keepalive.start()
+            return
+        elif message['MessageType'] == "KeepAlive":
+            LOG.debug("KeepAlive received from server.")
+            return
+
         if data is None:
             data = {}
         elif type(data) is not dict:
@@ -79,7 +114,6 @@ class WSClient(threading.Thread):
 
         if message['MessageType'] in ('RefreshProgress',):
             LOG.debug("Ignoring %s", message)
-
             return
 
         if not self.client.config.data['app.default']:
@@ -91,5 +125,12 @@ class WSClient(threading.Thread):
 
         self.stop = True
 
+        if self.keepalive is not None:
+            self.keepalive.stop()
+
         if self.wsc is not None:
             self.wsc.close()
+
+        if not self.multi_client:
+            self.global_stop = True
+            self.global_wsc = None
